#include "acedatatype.h"
#include "maxon/datadescription_ui.h"
#include "customgui_acedatatype.h"
#include "lib_description.h"

namespace maxonsdk
{

AceDataType::AceDataType(Float value) : _value(value)
{

}

maxon::Result<void> AceDataType::DescribeIO(const maxon::DataSerializeInterface& stream)
{
	iferr_scope;

	PrepareDescribe(stream, AceDataType);

	Describe("_value", _value, Float, maxon::DESCRIBEFLAGS::NONE) iferr_return;

	return maxon::OK;
}

Float AceDataType::GetValue() const
{
	return _value;
}

void AceDataType::SetValue(Float value)
{
	_value = value;
}

MAXON_DATATYPE_REGISTER(AceDataType);
MAXON_DEPENDENCY_REGISTER(AceDataTypeRegistration);

Int32 AceDataTypeClass::GetId()
{
	return ACEDATATYPE_ID;
}

CustomDataType* AceDataTypeClass::AllocData()
{
	AceDataType* data = NewObjClear(AceDataType);
	return data;
}

void AceDataTypeClass::FreeData(CustomDataType* data)
{
	DeleteObj(data);
}

Bool AceDataTypeClass::CopyData(const CustomDataType* src, CustomDataType* dst, AliasTrans* aliastrans)
{
	const AceDataType& srcData = *static_cast<const AceDataType*>(src);
	AceDataType& dstData = *static_cast<AceDataType*>(dst);
	dstData = srcData;
	return true;
}

Int32 AceDataTypeClass::Compare(const CustomDataType* a, const CustomDataType* b)
{
	const	AceDataType& dataA = *static_cast<const	AceDataType*>(a);
	const	AceDataType& dataB = *static_cast<const	AceDataType*>(b);

	if (dataA == dataB)
		return 0;
	if (dataA < dataB)
		return -1;
	return 1;
}

Bool AceDataTypeClass::WriteData(const CustomDataType* t_d, HyperFile* hf)
{
	const AceDataType& data = *static_cast<const AceDataType*>(t_d);
	return hf->WriteFloat(data.GetValue());
}

Bool AceDataTypeClass::ReadData(CustomDataType* t_d, HyperFile* hf, Int32 level)
{
	AceDataType& data = *static_cast<AceDataType*>(t_d);

	if (level > 0)
	{
		Float value;
		const Bool readSuccess = hf->ReadFloat(&value);
		if (readSuccess == false)
			return false;
		data.SetValue(value);
	}
	return true;
}

Bool AceDataTypeClass::InterpolateKeys(GeData& res, const GeData& t_dataA, const GeData& t_dataB, Float mix, Int32 flags)
{
	const AceDataType& dataA = *static_cast<const AceDataType*>(t_dataA.GetCustomDataType(ACEDATATYPE_ID));
	const AceDataType& dataB = *static_cast<const AceDataType*>(t_dataB.GetCustomDataType(ACEDATATYPE_ID));

	const Float valueA = dataA.GetValue();
	const Float valueB = dataB.GetValue();
	const AceDataType mixedValue = AceDataType(maxon::Blend(valueA, valueB, mix));
	res = GeData(ACEDATATYPE_ID, (CustomDataType&)mixedValue);
	return true;
}

const Char* AceDataTypeClass::GetResourceSym()
{
	return "ACEDATATYPE";
}

void AceDataTypeClass::GetDefaultProperties(BaseContainer& data)
{
	data.SetInt32(DESC_CUSTOMGUI, ACEDATATYPEGUI_ID);
	data.SetInt32(DESC_ANIMATE, DESC_ANIMATE_ON);
}

maxon::Result<void> AceDataTypeClass::RegisterDataType()
{
	iferr_scope;

	static const Int32 version = 1;
	const Bool isDataTypeRegistered = RegisterCustomDataTypePlugin("AceDataType"_s, CUSTOMDATATYPE_INFO_LOADSAVE, NewObjClear(AceDataTypeClass), version);
	CheckState(isDataTypeRegistered == true);

	return maxon::OK;
}

} // namespace maxonsdk