#include "customgui_acedatatype.h"

#include "maxon/datadescription.h"

#include "lib_description.h"
#include "c4d_customdatatypeplugin.h"


#include "maxon/datadescription_data.h"
#include "maxon/datadescription_ui.h"
#include "maxon/desctranslation.h"
#include "c4d_basedocument.h"
#include "c4d_colors.h"

#include "customgui_helpers.h"

namespace maxonsdk
{

#define ACEDATATYPEGUI_RESOURCE_SYMBOL "ACEDATATYPEGUI"
static Int32 g_resourceDataTypeTable[] = { ACEDATATYPE_ID };

Int32 AceDataTypeData::GetId()
{
	return ACEDATATYPEGUI_ID;
}
CDialog* AceDataTypeData::Alloc(const BaseContainer &settings)
{
	AceDataTypeGui *dlg = NewObjClear(AceDataTypeGui, settings, GetPlugin());
	if (!dlg)
		return nullptr;

	CDialog *cdlg = dlg->Get();
	if (!cdlg)
		return nullptr;

	return cdlg;
}
void AceDataTypeData::Free(CDialog *dlg, void *userdata)
{
	if (!dlg || !userdata)
		return;
	AceDataTypeGui *sub = (AceDataTypeGui*)userdata;
	DeleteObj(sub);
}
const Char* AceDataTypeData::GetResourceSym()
{
	return ACEDATATYPEGUI_RESOURCE_SYMBOL;
}
CustomProperty* AceDataTypeData::GetProperties()
{
	return nullptr;
}
Int32 AceDataTypeData::GetResourceDataType(Int32 *&table)
{
	table = g_resourceDataTypeTable;
	return sizeof(g_resourceDataTypeTable) / sizeof(Int32);
}

static AceDataTypeLib g_aceDataTypeGuiLib;
maxon::Result<void> AceDataTypeData::RegisterGuiPlugin()
{
	iferr_scope;

	ClearMem(&g_aceDataTypeGuiLib, sizeof(g_aceDataTypeGuiLib));
	FillBaseCustomGui(g_aceDataTypeGuiLib);

	const Bool isLibraryInstalled = InstallLibrary(ACEDATATYPEGUI_ID, &g_aceDataTypeGuiLib, 1000, sizeof(g_aceDataTypeGuiLib));
	CheckState(isLibraryInstalled == true);

	const Bool isGuiRegistered = RegisterCustomGuiPlugin("AceDataTypeGui"_s, CUSTOMGUI_SUPPORT_LAYOUTDATA | CUSTOMGUI_DISALLOW_TAKESOVERRIDE, NewObjClear(AceDataTypeData));
	CheckState(isGuiRegistered == true);

	return maxon::OK;
}

AceDataTypeGui::AceDataTypeGui(const BaseContainer &settings, CUSTOMGUIPLUGIN *plugin) : iBaseCustomGui(settings, plugin)
{
	const GeData& data = settings.GetData(DESC_DEFAULT);
	if (data.GetType() == ACEDATATYPE_ID)
	{
		_defaultValue = *static_cast<const AceDataType*>(data.GetCustomDataType(ACEDATATYPE_ID));
	}
}

#define NUMBERFIELD_ID 1001
Bool AceDataTypeGui::CreateLayout()
{
	static Int32 EDITW = 100;
	static Int32 EDITH = 10;

	GroupBegin(0, BFH_LEFT | BFV_CENTER, 0, 1, String(), 0);
	C4DGadget* field = AddEditNumberArrows(NUMBERFIELD_ID, 0, EDITW, EDITH);
	SetDefaultColor(field, COLOR_BGEDIT, Vector(1, 0, 0));
	GroupEnd();

	return iCustomGui::CreateLayout();
}

Bool AceDataTypeGui::InitValues()
{
	const Float floatValue = _value.GetValue();
	SetFloat(NUMBERFIELD_ID, floatValue);
	return true;
}

Bool AceDataTypeGui::Command(Int32 id, const BaseContainer &msg)
{
	Bool handled = true;
	switch (id)
	{
		case NUMBERFIELD_ID:
		{
			if (msg.GetBool(BFM_ACTION_RESET) == true)
			{
				_value = _defaultValue;
			}
			else
			{
				const Float floatValue = msg.GetFloat(BFM_ACTION_VALUE);
				_value = AceDataType(floatValue);
			}
			break;
		}
		default:
		{
			handled = false;
			break;
		}
	}

	if (handled)
	{
		BaseContainer m(msg);
		m.SetInt32(BFM_ACTION_ID, GetId());
		m.SetData(BFM_ACTION_VALUE, GeData(ACEDATATYPE_ID, (const CustomDataType&)_value));
		SendParentMessage(m);
	}

	return iCustomGui::Command(id, msg);
}

TriState<GeData> AceDataTypeGui::GetData()
{
	TriState<GeData> triState;
	triState.Add(GeData(ACEDATATYPE_ID, (const CustomDataType&)_value));
	return triState;
}

Bool AceDataTypeGui::SetData(const TriState<GeData> &tristate)
{
	const GeData& data = tristate.GetValue();
	if (data.GetType() != ACEDATATYPE_ID)
		return false;

	const AceDataType& value = *static_cast<const AceDataType*>(tristate.GetValue().GetCustomDataType(ACEDATATYPE_ID));
	_value = value;
	InitValues();
	return true;
}

maxon::Result<void> AceDataTypeGuiConversion::Initialize()
{
	iferr_scope;

	// Initialization of the default conversions on startup.
	// Here one can define which ui should appear for which data type if nothing is declared in the description.
	maxon::UiConversionInterface::AddDefaultConversion(maxon::GetDataType<maxonsdk::AceDataType>(), AceDataTypeGuiConversion::GetClass().GetId()) iferr_return;

	return maxon::OK;
}

maxon::Result<void> AceDataTypeGuiConversion::QuerySupportedDataTypes(maxon::BaseArray<maxon::DataType>& dataTypes) const
{
	iferr_scope;
	dataTypes.Append(maxon::GetDataType<maxonsdk::AceDataType>()) iferr_return;

	return maxon::OK;
}

maxon::Result<void> AceDataTypeGuiConversion::CreateC4DDescription(const maxon::DataType& dataType, Description& c4dDescription, const maxon::LanguageRef& language,
	const maxon::DataDictionary& dataEntry, const maxon::DataDictionary& guiEntry, const maxon::DataDescription& mainDataDescription, const maxon::DataDescription& stringDescription, const DescID& mainId,
	const DescID& groupId, const maxon::PatchC4DDescriptionEntryDelegate& patchEntryFunc, maxon::DescTranslation& translateIds,
	const maxon::BaseArray<maxon::InternedId>& parentIds, const DescID& parentFoldId, const maxon::GetDataCallbackType& getDataCallback,
	const maxon::GetExtraDataCallbackType& getExtraDataDelegate) const
{
	iferr_scope;

	static const Int dtype = ACEDATATYPE_ID;
	DescID descId(mainId);
	descId.PushId(DescLevel(1, dtype, 0));

	BaseContainer param = GetCustomDataTypeDefault(dtype);
	patchEntryFunc(param) iferr_return;

	AceDataType defaultData = dataEntry.Get(maxon::DESCRIPTION::DATA::BASE::DEFAULTVALUE.Get(), AceDataType());
	GeData value;
	ConvertToC4D(value, dataType, maxon::Data(defaultData), DescID(), dataEntry, guiEntry, maxon::GetExtraDataCallbackType()) iferr_return;
	param.SetData(DESC_DEFAULT, value);
	// We do not have to set DESC_CUSTOMGUI explicitly, because set up in AceDataTypeClass::GetDefaultProperties(). 

	// We have set up our custom gui and embed it into the Attribute Manager for nodes.
	CustomGuiHelpers::SetPDescription(c4dDescription, translateIds, descId, param, groupId, dataEntry, guiEntry, parentIds) iferr_return;

	return maxon::OK;
}

maxon::Result<void> AceDataTypeGuiConversion::ConvertToC4D(GeData& output, const maxon::DataType& dataType, const maxon::Data& data, const DescID& descIdSuffix,
	const maxon::DataDictionary& dataEntry, const maxon::DataDictionary& guiEntry, const maxon::GetExtraDataCallbackType& extraDataDelegate) const
{
	iferr_scope;

	// We transport the data type from 'new' to 'classical', i.e. from node to gui.
	AceDataType value = data.Get<AceDataType>() iferr_return;
	output = GeData(ACEDATATYPE_ID, (CustomDataType&)value);
	return maxon::OK;
}

maxon::Result<maxon::Tuple<maxon::Data, maxon::Bool>> AceDataTypeGuiConversion::ConvertToCore(const maxon::DataType& dataType, const GeData& data, const DescID& descIdSuffix,
	const maxon::DataDictionary& dataEntry, const maxon::DataDictionary& guiEntry, const maxon::Data& oldData,
	const maxon::GetExtraDataCallbackType& extraDataDelegate) const
{
	// We transport the data type from 'classical' to 'new', i.e. from gui to node.
	const AceDataType& value = *static_cast<const AceDataType*>(data.GetCustomDataType(ACEDATATYPE_ID));
	return maxon::Tuple<maxon::Data, maxon::Bool>(maxon::Data(AceDataType(value)), false);
}

MAXON_COMPONENT_OBJECT_REGISTER(AceDataTypeGuiConversion, maxon::UiConversions, "net.maxonsdk.ui.acedatatypeguiconversion");


// MAXON_DECLARATION_REGISTER(maxon::DataTypeBuilderRegistry, "net.maxonsdk.datatype.acedatatype")
// {
// 	// Show this string in the GUI.
// 	return "maxonsdk::AceDataType"_cs;
// }

} // namespace maxonsdk
